/*
 *  Kylin security devctl control（模块，包功能介绍）
 *
 *  This file contains the definitions needed for Kylin security devctl
 *  functions
 *
 *  Authors:  WKL <wangkanglong@kylinos.cn>
 *
 *  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#ifndef KYSEC_DEVCTL_H
#define KYSEC_DEVCTL_H

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <fcntl.h>

#define DEVNAME_MAX 20
#define CHAR_MID_LEN 128
#define CHAR_MAX_LEN 256
#define DEVCTL_KERNEL_STRUCT 1
#define DEVCTL_SET_PERM_DENY	-5
#define PORT_LEVEL_MAX 5

/**  设备类型枚举 */
enum kysec_dev_type {
	KYSEC_USB = 1,							///< usb接口
	KYSEC_SERIAL,							///< 串口
	KYSEC_PARALLEL,							///< 并口
	KYSEC_TYPEC,							///< type-c接口
	KYSEC_CDROM,							///< 光驱（内置光驱）
	KYSEC_PRINTER,							///< 打印机类
	KYSEC_WIRELESS,							///< 无线网络
	KYSEC_ETHERNET,							///< 有线网络
	KYSEC_BLUETOOTH,						///< 蓝牙
	KYSEC_CAMERA,							///< 摄像头
	KYSEC_SCANNER,							///< 扫描仪
	KYSEC_AUDIO,							///< 音频(目前仅管控到3.5mm jack)
	KYSEC_IEEE1394,							///< IEEE1394接口
	KYSEC_PCMCIA,							///< PCMCIA接口
	KYSEC_PCIE,								///< PCIE接口
	KYSEC_SATA,								///< SATA接口
	KYSEC_PS2,								///< PS2接口 （暂未提供）
	KYSEC_VGA,								///< VGA接口 （暂未提供）
	KYSEC_HDMI,								///< HDMI接口
	KYSEC_DP,								///< DP接口
	KYSEC_INFRARED,							///< 红外设备
	KYSEC_MODEM,							///< 网络modem
	KYSEC_MMC,								///< mmc设备
	KYSEC_SCSI_RAID,						///< scsi_raid设备
	KYSEC_FLOPPY_DRIVE,						///< 软驱设备
	KYSEC_VIDEO,							///< 图像设备
	KYSEC_DIGITAL_CAMERA,					///< 数码相机设备
	KYSEC_MULTI_SERIAL_CARD,				///< 多串口
	KYSEC_WINCE,							///< wince设备(手机、ipad等)
	KYSEC_PRINTER_LOCAL,					///< 本地打印机
	KYSEC_PRINTER_NET,						///< 网络打印机
	KYSEC_FILE_SHARING,						///< 暂未使用（sm）
	KYSEC_BATTERY_AND_UPS,					///< 暂未使用（sm）
	KYSEC_BIOMETRIC,						///< 暂未使用（sm）
	KYSEC_DISK_DRIVE,						///< 暂未使用（sm）
	KYSEC_DISPLAY_CARD,						///< 暂未使用（sm）
	KYSEC_HDC,								///< 暂未使用（sm）
	KYSEC_KEYBOARD,							///< 暂未使用（sm）
	KYSEC_MOUSE,							///< 暂未使用（sm）
	KYSEC_STORAGE_VOLUME,					///< 暂未使用（sm）
	KYSEC_CDDRIVE,							///< 暂未使用（sm）
	KYSEC_VOLUME_SHADOW,					///< 卷影设备 暂未使用（sm）
	KYSEC_TAPE,								///< 磁带机 暂未使用（sm）
	KYSEC_DEV_MAX
};

//usb设备的类型
enum kysec_usb_type {
	KYSEC_HARD_DRIVE = 1,					///< u盘 (属于0x08类)
	KYSEC_USB_CDROM,						///< usb光驱 (属于0x08类)
	KYSEC_USB_PRINTER,						///< 0x07 usb打印机
	KYSEC_USB_CAMERA,						///< 0x0e usb video设备
	KYSEC_USB_BLUETOOTH,					///< 0xe0 无线控制器、usb蓝牙
	KYSEC_USB_STORAGE,						///< 0x08 大容量存储
	KYSEC_USB_PER_INTERFACE,				///< 0x00 保留分类
	KYSEC_USB_AUDIO,						///< 0x01 音频设备
	KYSEC_USB_COMM,							///< 0x02 通信及通信类控制设备
	KYSEC_USB_HID,							///< 0x03 人体工学设备
	KYSEC_USB_PHYSICAL,						///< 0x05 物理设备
	KYSEC_USB_STILL_IMAGE,					///< 0x06 图像
	KYSEC_USB_HUB,							///< 0x09 hub设备
	KYSEC_USB_CDC_DATA,						///< 0x0a 通信类数据设备
	KYSEC_USB_CSCID,						///< 0x0b 智能卡
	KYSEC_USB_CONTENT_SEC,					///< 0x0d 内容安全
	KYSEC_USB_MISC,							///< 0xef 其他设备
	KYSEC_USB_APP_SPEC,						///< 0xfe 应用程序
	KYSEC_USB_VENDOR_SPEC,					///< 0xff 厂家自动以
	KYSEC_USB_PERSONAL_HEALTH,				///< 0x0f 个人医疗(linux内核没有该类型，为兼容windows)
	KYSEC_USB_DIAGNOSTIC,					///< 0xdc 诊断设备(linux内核没有该类型，为兼容windows)
	KYSEC_USB_MAX
};

/**  黑名单类型枚举 */
enum kysec_devctl_blacklist_type {
	WIRELESS_SSID = 1,						///< 热点黑名单类型
	NET_CARD,								///< 网卡黑名单类型
	TYPE_MAX
};

/**  设备权限枚举 */
enum kysec_devctl_perm {
	KYSEC_DEVCTL_ENABLE = 1,						///< 启用
	KYSEC_DEVCTL_DISABLE,							///< 禁用
	KYSEC_DEVCTL_RO = 5,							///< 只读权限
	KYSEC_DEVCTL_PERM_MAX
};

/**  设备管控同步内核的策略结构体（旧版本） */
struct kysec_devctl_kernel_old {
	int type;								///< 设备类型(见kysec_dev_type枚举)
	int usb_subtype;						///< usb设备的类型(见kysec_usb_type枚举)
	unsigned int pid;						///< usb设备的pid
	unsigned int vid;						///< usb设备的vid
	int perm;								///< 设备权限(见kysec_devctl_perm枚举)
	char serial[CHAR_MID_LEN];				///< usb设备的序列号
};

/**  设备管控同步内核的策略结构体 */
struct kysec_devctl_kernel {
	int type;								///< 设备类型(见kysec_dev_type枚举)
	int usb_subtype;						///< usb设备的类型(见kysec_usb_type枚举)
	unsigned int pid;						///< usb设备的pid
	unsigned int vid;						///< usb设备的vid
	int perm;								///< 设备权限(见kysec_devctl_perm枚举)
	int special_type;						///< usb设备的usb_type(见kysec_usb_type枚举)
	char serial[CHAR_MID_LEN];				///< usb设备的序列号
	int version;							///< 版本号
};

/**  设备管控策略结构体 */
struct kysec_devctl_info {
	int type;								///< 设备类型(见kysec_dev_type枚举)
	int usb_subtype;						///< usb设备的类型(见kysec_usb_type枚举)
	unsigned int pid;						///< usb设备的pid
	unsigned int vid;						///< usb设备的vid
	int perm;								///< 设备权限(见kysec_devctl_perm枚举)
	int oldperm;							///< 设备旧权限(见kysec_devctl_perm枚举，安全中心交互)
	char serial[CHAR_MID_LEN];				///< usb设备的序列号
	int special_type;						///< usb设备的usb_type(见kysec_usb_type枚举)
	char displayname[CHAR_MAX_LEN];			///< 设备的显示名称 (安全中心交互)
};

/**  设备管控sys文件数据的结构体（usb设备） */
struct kysec_devctl_sysf {
	int dtype;								///< usb设备的类型(见kysec_usb_type枚举)
	unsigned int devpid;					///< usb设备的pid
	unsigned int devvid;					///< usb设备的vid
	int devperm;							///< 内核中设备的权限(见kysec_devctl_perm枚举)
	char product[CHAR_MID_LEN];				///< 产品名
	char manufacturer[CHAR_MID_LEN];		///< 厂商名
	char serial[CHAR_MID_LEN];				///< usb设备的序列号
	char dev_name[DEVNAME_MAX];				///< sys下的文件节点
	char displayname[CHAR_MAX_LEN];			///< 设备的显示名称（“厂商名_产品名”格式）
};

/**  设备的连接记录 */
struct device_record {
	int type;								///< usb设备的类型(见kysec_usb_type枚举)
	int perm;								///< 设备权限(见kysec_devctl_perm枚举)
	unsigned int pid;						///< usb设备的pid
	unsigned int vid;						///< usb设备的vid
	char serial[CHAR_MID_LEN];				///< usb设备的序列号
	char access_t[CHAR_MID_LEN];			///< usb设备的接入时间
	char duration[20];						///< usb设备的连接时长
	char displayname[CHAR_MAX_LEN];			///< usb设备的显示名称
};

/**  黑名单策略的结构体 */
struct kysec_devctl_blacklist {
	char name[CHAR_MID_LEN];				///< 黑名单名称
	int type;								///< 黑名单类型(见kysec_devctl_blacklist_type枚举)
};

/**  usb接口的端口信息(bios管控) */
struct port_info {
	int port[PORT_LEVEL_MAX];				///< 端口号
	int port_level;							///< 端口层级
	unsigned char pci_bus;					///< 端口控制器所属的pci_bus
	unsigned char pci_dev;					///< 端口控制器所属的pci_dev
	unsigned char pci_func;					///< 端口控制器所属的pci_func
	int usb_subtype_perm[KYSEC_USB_MAX];	///< 端口的权限集
	int priority;							///< 端口权限的优先级
};

struct kysec_bios_devctl_info {
	struct port_info info;					///< usb接口的端口信息结构体
	int pci_flag;							///< 标志位（是否匹配pci）
	int version;							///< 版本号
};

/**  热点黑名单策略的结构体（旧接口，已弃用） */
struct kysec_devctl_ssid {
	char blkssid[CHAR_MID_LEN];				///< 热点名
};

#ifdef __cplusplus
extern "C" {
#endif
/**
 * @brief 添加设备管控策略
 * @param  {struct kysec_devctl_info} *recv(策略结构体)
 * @return {int} 0：成功，非0：失败（-5：禁止添加，-1014：重复添加，-1016：策略同步内核失败，-1017：写配置文件失败，-1018:无效的策略）
 */
int kysec_devctl_add_perm(struct kysec_devctl_info *recv);

/**
 * @brief 删除设备管控策略
 * @param  {struct kysec_devctl_info} *recv(策略结构体)
 * @return {int} 0：成功，非0：失败（-1016：策略同步内核失败，-1017：写配置文件失败，-1018:无效的策略）
 */
int kysec_devctl_remove_perm(struct kysec_devctl_info *recv);

/**
 * @brief 更新设备管控策略
 * @param  {struct kysec_devctl_info} *recv(策略结构体)
 * @return {int} 0：成功，非0：失败（-5：禁止更新，-1015：重复更新，-1016：策略同步内核失败，-1017：写配置文件失败，-1018:无效的策略）
 */
int kysec_devctl_update_perm(struct kysec_devctl_info *recv);

/**
 * @brief 添加黑名单策略
 * @param  {struct kysec_devctl_blacklist} *recv(策略结构体)
 * @return {int} 0：成功，非0：失败（-1014：重复添加，-1016：策略同步内核失败，-1017：写配置文件失败，-1018:无效的策略）
 */
int kysec_devctl_blacklist_add(struct kysec_devctl_blacklist *recv);

/**
 * @brief 删除黑名单策略
 * @param  {struct kysec_devctl_blacklist} *recv(策略结构体)
 * @return {int} 0：成功，非0：失败（-1016：策略同步内核失败，-1017：写配置文件失败，-1018:无效的策略）
 */
int kysec_devctl_blacklist_del(struct kysec_devctl_blacklist *recv);

/**
 * @brief 批量添加设备管控策略
 * @param  {struct kysec_devctl_info} *data(策略结构体)
 * @param  {int} count(策略个数)
 * @return {int} 成功添加的策略个数
 */
int kysec_devctl_batch_add_perm(struct kysec_devctl_info *data, int count);

/**
 * @brief 获取所有设备管控的策略
 * @param  {int} *count（传出参数，策略数）
 * @return {struct kysec_devctl_info} 结构体
 */
struct kysec_devctl_info *kysec_devctl_read_all_dev_data(int *count);

/**
 * @brief 获取所有黑名单的策略
 * @param  {int} *count（传出参数，策略数）
 * @return {struct kysec_devctl_blacklist} 结构体
 */
struct kysec_devctl_blacklist *kysec_devctl_read_all_blacklist(int *count);

/**
 * @brief 获取所有接口/设备类的管控策略
 * @param  {int} *count（传出参数，策略数）
 * @return {struct kysec_devctl_info} 结构体
 */
struct kysec_devctl_info *kysec_devctl_read_not_special_data(int *count);

/**
 * @brief 获取所有special设备的管控策略
 * @param  {int} *count（传出参数，策略数）
 * @return {struct kysec_devctl_info} 结构体
 */
struct kysec_devctl_info *kysec_devctl_read_special_xmldata(int *count);

/**
 * @brief 获取sys下的设备信息文件
 * @param  {int} *count（传出参数,设备个数）
 * @return {struct kysec_devctl_sysf} 结构体
 */
struct kysec_devctl_sysf *kysec_devctl_read_sys_devinfo(int *count);

/**
 * @brief 管控策略及时生效
 * @param  {struct kysec_devctl_info} *recv(策略结构体)
 * @return {int} 0：成功，非0：失败
 */
int kysec_devctl_auto_sync_config(struct kysec_devctl_kernel *recv);

/**
 * @brief 获取60天以内的设备连接记录
 * @param  {int} *count（传出参数,记录条数）
 * @return {struct device_record} 结构体
 */
struct device_record *device_record_get(int *count);

/**
 * @brief 写入设备的连接记录
 * @param  {struct device_record} *record 设备连接记录
 * @param  {int} count（记录条数）
 * @return {int} 0:成功，-1：出错
 */
int kysec_devctl_xdg_insert_record(struct device_record *record, int count);

/**
 * @brief 切换设备管控状态
 * @param  {int} status 状态，0:关闭，1:开启
 * @return {int} 0:成功，-1：出错
 */
int kysec_devctl_switch_status(int status);

/**
 * @brief rebind usb设备
 * @param  {int} usb_type usb设备类型
 * @return {int} 0:成功，-1：出错
 */
int kysec_devctl_rebind_usb_device(int usb_type);

/**
 * @brief 开机阶段自动重新挂载usb设备
 * @return {int} 0:成功，-1：出错
 */
int kysec_devctl_rebind_usb_device_auto();

/**
 * @name: kysec_devctl_switch_bios_usb_status
 * @Descripttion: 修改config文件中的bios_usb管控状态
 * @msg:
 * @param {int} status 状态，0关闭，1开启
 * @return {*} 0:成功，-1：出错
 */
int kysec_devctl_switch_bios_usb_status(int status);

/**
 * @name: kysec_devctl_get_bios_usb_status
 * @Descripttion: 获取config文件中的bios_usb管控状态
 * @msg:
 * @return {*} 0:关闭，1：开启，其他：出错
 */
int kysec_devctl_get_bios_usb_status();

/**
 * @brief 获取sm的状态
 * @return {int} 0: 未启用， 非0： 启用
 */
int kysec_devctl_get_sm();

/**
 * @brief 清空所有策略（设备管控、黑名单、程序黑名单）
 */
void kysec_devctl_clean_all_policies();

/**
 * @brief 加载ksc的设备管控策略
 */
void kysec_devctl_load_ksc_policies();

/**
 * @brief 判断dir中是否存在file_name的文件
 * @param  {char} *dir
 * @param  {char} *file_name
 * @return {*} 1：存在，0：不存在
 */
int judeg_file_existd_dir(const char *dir, const char *file_name);

/**
 * @brief 清空所有设备管控策略
 */
void kysec_devctl_clean_all_device_policies();

/**
 * @brief 清空所有黑名单策略
 */
void kysec_devctl_clean_all_blacklist_policies();

/**
 * @brief 设备管控（黑名单）同步策略到内核
 * @param  {int} 操作类型
 * @param  {void *} recv（策略结构体）
 * @return {int} 0：成功，非0：失败
 */
int devctl_communicate_with_kernel(int op_type, void *recv);

/**
 * @brief 同步bios_usb管控策略
 * @return {int} 0：成功，非0：失败
 */
int kysec_devctl_sync_bios_usb_rules();

 /**
  * @name: kysec_devctl_read_all_ssid_data (弃用)
  * @Descripttion: 获取所有的热点黑名单
  * @msg:
  * @param {int} *count  黑名单的个数
  * @return {*}
  */

/**
 * @brief （弃用）获取所有的热点黑名单策略
 * @param  {int} *count（传出参数,策略条数）
 * @return {struct kysec_devctl_ssid} 结构体
 */
struct kysec_devctl_ssid *kysec_devctl_read_all_ssid_data(int *count);

/**
 * @brief 检查bios-usb管控下，该usb类型设备是否需要rebind
 * @param  {int} usb_type（usb设备类型）
 * @return {int} 1: 需要rebind， 其他：不需要
 */
int kysec_bios_usb_need_rebind_check(int usb_type);

/**
 * @brief 设置usb设备的默认权限
 * @param  {int} perm 默认权限值（见kysec_devctl_perm枚举）
 * @return {int} 0：成功，非0：失败（-1018:无效的权限值）
 */
int kysec_devctl_usb_set_default_perm(int perm);

#ifdef __cplusplus
}
#endif

#endif /* KYSEC_DEVCTL_H */
